pub type MatrixReport = Vec<Vec<String>>;
pub type SingleRow = std::iter::Once<Vec<String>>;

use super::{row_builder::RowBuilder, BenchmarkResult};
use std::time::Duration;

pub fn create_header() -> std::iter::Once<Vec<String>> {
    std::iter::once(vec![
        "Day".to_string(),
        "Task".to_string(),
        "How long (Seconds:Mili)".to_string(),
        "Expected".to_string(),
        "Actual".to_string(),
        "Used input file".to_string(),
    ])
}

pub fn create_sum_row(sum: Duration) -> SingleRow {
    std::iter::once(RowBuilder::total(sum).into_row())
}

pub fn create_average_row(average: Duration) -> SingleRow {
    std::iter::once(RowBuilder::average(average).into_row())
}

pub fn create_rows_for_every_solutions(
    loaded: impl IntoIterator<Item = BenchmarkResult>,
    mut on_ok: impl FnMut(Duration),
) -> MatrixReport {
    loaded
        .into_iter()
        .map(|result| {
            result
                .inspect(|on_success| on_ok(on_success.how_long()))
                .map(RowBuilder::from)
                .unwrap_or_else(RowBuilder::from)
        })
        .map(RowBuilder::into_row)
        .collect()
}
